/*
 * Copyright (C) 1998 LAAS/CNRS -- RIA --
 *
 * Maher Khatib - Tue Feb  3 1998
 *
 */
#ifndef _SICK_STR_H_
#define _SICK_STR_H_ 

/* #include "t3d.h" */

// DO NOT #include "pomStruct.h"
// because it has to be protected by the "import from pom" 
// clause in sick.gen


/* Usefull flag */
typedef enum SICK_BOOL {
  SICK_FALSE,
  SICK_TRUE
} SICK_BOOL;

typedef struct SICK_STRING {
  char name[64];
} SICK_STRING;

/* Usefull flag */
typedef enum SICK_FRAME {
  SICK_SICK_FRAME,
  SICK_ROBOT_FRAME,
  SICK_GLOBAL_FRAME
} SICK_FRAME;

/* Init request parameter */
typedef enum SICK_ACQUISITION_WAY {
  SICK_ACQ_REAL,
  SICK_ACQ_SIMUL_ENV,
  SICK_ACQ_REPLAY_FILES
} SICK_ACQUISITION_WAY;

typedef struct SICK_EXPORT_DATA {
  SICK_BOOL exportPolarPoints;
  SICK_BOOL exportCartesianPoints;
  SICK_BOOL exportSegments;
  SICK_FRAME cartesianFrame; /* for cartesian points AND segments */
} SICK_EXPORT_DATA;

typedef struct SICK_INIT_PARAMS {
  SICK_ACQUISITION_WAY acqWay;
  int unused;
  SICK_EXPORT_DATA exportData;
  SICK_STRING sickPosPosterName; /* usefull only in SICK_ACQ_REAL mode */
  SICK_STRING serialPort;
} SICK_INIT_PARAMS;

/* sick running status */
typedef enum sick_scan_task_state {
  SICK_FREE_STATE,
  SICK_WAITING_RESPONSE_STATE,
  SICK_RESET_STATE,
  SICK_READ_STATUS_STATE,
  SICK_READ_CONFIG_STATE,
  SICK_CONFIG_MODE_STATE,
  SICK_SET_CONFIG_STATE,
  SICK_SET_MEASURE_UNIT_STATE,
  SICK_SET_SCANNING_PARAMETERS_STATE,
  SICK_SET_BAUD_RATE_STATE,
  SICK_SET_SCAN_ON_RQST_STATE,
  SICK_SET_SCAN_CONT_STATE,
  SICK_CONFIGURATED_STATE
} SICK_SCAN_TASK_STATE;

/* sick mode */
typedef enum sick_control_modes {
  SICK_CONFIGURATION_MODE,
  SICK_SCAN_ON_RQST_MODE,
  SICK_SCAN_CONTINUOUSLY_MODE,
  SICK_SCAN_CONTINUOUSLY_ON_RQST_MODE
} SICK_CONTROL_MODES;

/* sick mode */
typedef enum sick_scan_modes {
  SICK_PERIODIC_SCAN,
  SICK_CONTINUOUS_SCAN
} SICK_SCAN_MODES;

/* sick scanning range parameters */
typedef enum sick_scanning_angle_range {
  SICK_SCAN_100_DEG,
  SICK_SCAN_180_DEG
} SICK_SCANNING_ANGLE_RANGE;

/* sick scanning resolution parameters */
typedef enum sick_scanning_angle_resolution {
  SICK_SCAN_ONE_DEG_RESOLUTION,
  SICK_SCAN_HALF_DEG_RESOLUTION
} SICK_SCANNING_ANGLE_RESOLUTION;

/* sick measure precision */
typedef enum sick_measure_length_resolution {
  SICK_MEASURE_IN_MM,
  SICK_MEASURE_IN_CM,
  SICK_MEASURE_IN_10CM
} SICK_MEASURE_LENGTH_RESOLUTION;

/* sick baud rate */
typedef enum sick_baud_rate {
  SICK_9600_BAUD,
  SICK_19200_BAUD,
  SICK_38400_BAUD
} SICK_BAUD_RATE;

/* sick global config (current state) */
typedef struct sick_config_str {
  SICK_CONTROL_MODES control_mode;
  SICK_SCANNING_ANGLE_RANGE angle_range;
  SICK_SCANNING_ANGLE_RESOLUTION angle_resolution;
  SICK_MEASURE_LENGTH_RESOLUTION length_resolution;
  SICK_BAUD_RATE baud;
  int length_range_max;   /* Depends on lengthResolution */
} SICK_CONFIG_STR;

/* sick global config (required config) */
typedef struct sick_user_str {
  SICK_SCANNING_ANGLE_RANGE angle_range;
  SICK_SCANNING_ANGLE_RESOLUTION angle_resolution;
  SICK_MEASURE_LENGTH_RESOLUTION length_resolution;
  SICK_BAUD_RATE baud;
} SICK_USER_STR;

/* robot speed */
typedef struct SICK_SPEED_STR {
  double v, w;
  double vx, vy;
} SICK_SPEED_STR;

/* sick measure header
   NOTE: send_time and rcv_time are system time in milliseconds. They
     are filled by calling sick_lib_mtime() of hardlib/sickLib, which
     calls clock_gettime() or gettimeofday() depending on the
     HAS_POSIX_CLOCK preprocessor symbol. */
typedef struct SICK_MEASURES_HEADER_STR {
  int np;				   /* number of points */
  SICK_FRAME frame;
  unsigned int send_time;		   /* start acquisition time */
  unsigned int rcv_time;		   /* start acquisition time */
  double theta_min;			   /* theta min */
  double dtheta;			   /* acquisition increment */
  POM_SENSOR_POS	sickPomPos;                /* All pose data */
} SICK_MEASURES_HEADER_STR;

/*
 * Sick cartesian point str 
 */
typedef struct SICK_POINT {
  double x, y;
} SICK_POINT;


/* max number of points */
#define SICK_N_PT_MAX                  (361)

typedef struct SICK_CART_MEASURES {
  SICK_POINT points[SICK_N_PT_MAX];
} SICK_CART_MEASURES;
  

/*
 * sick measures str
 */
typedef struct SICK_MEASURES_STR {
  SICK_MEASURES_HEADER_STR header;
  double rho [SICK_N_PT_MAX];
} SICK_MEASURES_STR;

typedef struct SICK_POLAR_MEASURES_STR {
  SICK_MEASURES_STR measures;
  double theta [SICK_N_PT_MAX];
} SICK_POLAR_MEASURES_STR;

typedef struct SICK_CART_MEASURES_STR {
  SICK_MEASURES_HEADER_STR header;
  SICK_POINT cartesian_pt [SICK_N_PT_MAX];
} SICK_CART_MEASURES_STR;


/*
 * sick parser control
 */

/* max length of cmds list */
#define SICK_PARSER_CMDS_MAX           (10)

typedef struct sick_parser_control_str {
  int time;			/* waiting time */
  int rep;			/* repetition */
  int ncmd;			/* number of command */
  int index;			/* index of current command */
  SICK_SCAN_TASK_STATE cmds[SICK_PARSER_CMDS_MAX];
  SICK_SCAN_TASK_STATE state, last;
} SICK_PARSER_CONTROL_STR;


/*
 * Where to save data
 */
#define SICK_MAX_STRLEN 256

typedef enum SICK_SAVE_MODE {
  SICK_SAVE_ONCE,           /* save on request */
  SICK_SAVE_CONTINUOUS,     /* save every new data */
  SICK_SAVE_CONTINUOUS_MOVE /* save every new data if robot pos changes */
} SICK_SAVE_MODE;

typedef struct SICK_SAVE_FILE_PARAMS {
  char baseName[SICK_MAX_STRLEN];	/* base directory */
  int num;				/* first image number (auto-incremented) */
  SICK_SAVE_MODE saveMode;
  int saveContinuousFrequency;    /* 1: save every acq; 
				     2: save every 2 acq; ... */
  int unused;
} SICK_SAVE_FILE_PARAMS;

/**----------------------------------------------------------------------------
 ** SEGMENTATION PARAMS
 **/

typedef struct SICK_SEG {
  double x1, y1;
  double x2, y2;
} SICK_SEG;

/* Parameters for segmentation (not used now) */
typedef struct SICK_SEG_PARAMS {
  SICK_BOOL getVariances;
  int nbMinSegPts;          /* Nb min pts in a seg */
  double leastSquareThres;  /* Threshold for the least square estimation */
  double distMaxSegPts;     /* Max dist between 2 points in a seg */
} SICK_SEG_PARAMS;


/** ------------------------------------------------------------
 ** DESCRIPTION OF SEGMENTS
 **/

typedef struct SICK_SEG_HEADER {
  int nbSegs;                    /* number of segments */
  int nbLonelyPoints;            /* number of points not in segment */
  SICK_FRAME frame;
  int unused;
  double dtheta;			   /* acquisition increment */
  double unused2;
  unsigned int sickTime;         /* Acquisition time */
  SICK_BOOL variancesUpdated;    /* Segment variances recorded or not */
  POM_SENSOR_POS sickPomPos;     /* Robot/sick pos during sick acquisition */
} SICK_SEG_HEADER;

/* Acq segs poster */
#define SICK_MAX_ACQ_SEGS 100

typedef struct SICK_SEGS {
  SICK_SEG segs[SICK_MAX_ACQ_SEGS];
} SICK_SEGS;

typedef struct SICK_LONELY_PTS {
  SICK_POINT point[SICK_N_PT_MAX];
} SICK_LONELY_PTS;

#ifdef SEG_VARIANCES
typedef struct SICK_VAR_SEG {
  double r, o;
  double vr, vo, vro;
} SICK_VAR_SEG;

typedef struct SICK_VAR_SEGS {
  SICK_VAR_SEG var[SICK_MAX_ACQ_SEGS];
} SICK_VAR_SEGS;

#endif /* SEG_VARIANCES */


#endif /* _SICK_STR_H_ */
